#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <time.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>
#include <X11/Xproto.h>

#include <string.h>
#include <ctype.h>

#include "tv-freq.h"
#include "fftv.h"
#include "tv-win.h"
#include "xkey.h"

int ffmpeg_server=0;

typedef struct {
  char *argname;
  char *argval;
} FFTV_CONF_S;

FFTV_CONF_S  *fftv_conf_arr;
int fftv_conf_arrN;

void free_fftv_conf_arr()
{
  int i;

  for(i=0; i<fftv_conf_arrN; i++) {
    free(fftv_conf_arr[i].argname);
    free(fftv_conf_arr[i].argval);
  }

  free(fftv_conf_arr); fftv_conf_arr=NULL;
  fftv_conf_arrN=0;
}


void set_FFTV_MODE_by_arg(char *exec_name)
{
#if 0
  char *p = exec_name + strlen(exec_name)-1;

  while (p > exec_name && *p!='/')
    p--;
  if (*p == '/')
    p++;
#else
  char *p = basename(exec_name);
#endif

  if (!strcmp(p,"fftv")) {
    set_FFTV_MODE_TV();
  } else
  if (!strcmp(p,"ffradio")) {
    set_FFTV_MODE_RADIO();
  } else
    return;
}

char *fftv_conf_file_path(char *fftv_states_dir, char *fftv_conf)
{
  char *fname;

  if (fftv_mode & FFTV_MODE_TV) {
    set_FFTV_MODE_TV();
    fname = "conf-tv";
  } else
  if (fftv_mode & FFTV_MODE_RADIO) {
    set_FFTV_MODE_RADIO();
    fname = "conf-radio";
  } else
    return NULL;

  get_fftv_states_dir(fftv_states_dir);
  sprintf(fftv_conf, "%s/%s",fftv_states_dir,fname);
  return fname;
}


static void incr_fftv_conf_arr()
{
  fftv_conf_arr=realloc(fftv_conf_arr, sizeof(FFTV_CONF_S) * (fftv_conf_arrN+1));
  bzero(&fftv_conf_arr[fftv_conf_arrN],sizeof(FFTV_CONF_S));
}

void save_arg_val_file()
{
  char fftv_conf[256], fftv_states_dir[256];

  fftv_conf_file_path(fftv_states_dir, fftv_conf);
  FILE *fp;

  if ((fp=fopen(fftv_conf,"w"))==NULL)
    return;

  int i;
  for(i=0; i<fftv_conf_arrN; i++) {
    fprintf(fp,"%s %s\n", fftv_conf_arr[i].argname,
      fftv_conf_arr[i].argval ? fftv_conf_arr[i].argval : "-");
  }

  fclose(fp);
}

static int filename_idx()
{
  int i;

  for(i=0; i < fftv_conf_arrN; i++) {
    if (fftv_conf_arr[i].argname[0]!='-')
      break;
  }

  return i;
}

static int argname_idx(char *argname)
{
    int i;

    for(i=0; i < fftv_conf_arrN; i++) {
      if (!strcmp(fftv_conf_arr[i].argname, argname))
        break;
    }

    return i;
}


void set_arg_val(char *argname, char *argval)
{
  int i;

  if (argname[0]=='-') {
    i = argname_idx(argname);

    if (i==fftv_conf_arrN) {
      i = filename_idx();

      incr_fftv_conf_arr();
      fftv_conf_arr[fftv_conf_arrN] = fftv_conf_arr[i];
      fftv_conf_arrN++;
    }
    else {
      free(fftv_conf_arr[i].argname);
      free(fftv_conf_arr[i].argval);
    }

    fftv_conf_arr[i].argname = strdup(argname);

    if (argval)
      fftv_conf_arr[i].argval = strdup(argval);
    else
      fftv_conf_arr[i].argval = NULL;
  } else {
    int i = filename_idx();

    if (i==fftv_conf_arrN) {
      incr_fftv_conf_arr();
      fftv_conf_arrN++;
    }
    else {
      free(fftv_conf_arr[i].argname);
    }

    fftv_conf_arr[i].argname = strdup(argname);
    fftv_conf_arr[i].argval = NULL;
  }

  save_arg_val_file();
}

void set_arg_val_int(char *argname, int val)
{
  char tt[256];

  sprintf(tt,"%d",val);
  set_arg_val(argname, tt);
}

void rm_arg(char *argname)
{
  int i;

  i = argname_idx(argname);

  if (i==fftv_conf_arrN)
    return;

  free(fftv_conf_arr[i].argname);
  free(fftv_conf_arr[i].argval);

  for(;i<fftv_conf_arrN-1;i++)
    fftv_conf_arr[i]=fftv_conf_arr[i+1];

  fftv_conf_arrN--;
}


char *get_arg_val(char *argname)
{
  int i;

  for(i=0; i < fftv_conf_arrN; i++) {
    if (!strcmp(fftv_conf_arr[i].argname, argname))
      return fftv_conf_arr[i].argval;
  }

  return NULL;
}

char *orig_output_filename;
void parse_image_size(int *sizeX, int *sizeY, char *argval);
void set_grab_frame_size(int sizeX, int sizeY);

void append_arg(int *r_argc, char ***r_argv, char *str)
{
  char **argv=*r_argv;
  int argc = *r_argc;

  argv = realloc(argv, sizeof(char *) * (argc+1));
  argv[argc++] = strdup(str);

  *r_argv = argv;
  *r_argc = argc;
}

int user_grab_sizeX, user_grab_sizeY;

void load_conf(int *r_argc, char ***r_argv, gboolean b_firsttime)
{
  char **argv=*r_argv;
  int argc=*r_argc;
  char *fname;
  char fftv_conf[256], fftv_states_dir[256];

  fname = fftv_conf_file_path(fftv_states_dir, fftv_conf);

  FILE *fp;

  if (access(fftv_conf, R_OK) < 0) {
    char cmd[256];
    struct stat st;

    if (stat(fftv_states_dir, &st)) {
      if (mkdir(fftv_states_dir, 0755)) {
        perror(fftv_states_dir);
      }
    }

    sprintf(cmd, "cp %s/%s %s", FFTV_CONF, fname, fftv_states_dir);
    dbg("exec %s\n", cmd);
    system(cmd);
  }

  if ((fp=fopen(fftv_conf,"r"))==NULL) {
    dbg("??? cannot open %s\n",fftv_conf);
    exit(-1);
  }


  free_fftv_conf_arr();

  char **my_argv = (char **) malloc(sizeof(char *));
  int my_argc=1;
  my_argv[0] = argv[0];

  if (!b_firsttime) {
    int i;

    for(i=1;i<argc;i++) {
      free(argv[i]);
    }

    free(argv);
  }


  while (!feof(fp)) {
    char argname[128], argval[128];

    argname[0]=argval[0] = 0;
    fscanf(fp, "%s %s\n", argname, argval);
//    dbg("%d zzzzzzzzzzzz %s %s\n",my_argc, argname,argval);
    if (!argname[0] && !argval[0])
      continue;

    incr_fftv_conf_arr();

    fftv_conf_arr[fftv_conf_arrN].argname = strdup(argname);
    fftv_conf_arr[fftv_conf_arrN].argval =strdup(argval);

    if (ffmpeg_server && strcmp(argname, "-b"))
      orig_output_filename = "http://localhost:8090/feed1.ffm";
    else
    if (argname[0] && argname[0]!='-') {
      orig_output_filename = fftv_conf_arr[fftv_conf_arrN].argname;
    } else {
      if (!strcmp(argname, "-s"))
        parse_image_size(&user_grab_sizeX, &user_grab_sizeY, argval);

      if (argname[0]) {
         append_arg(&my_argc, &my_argv, argname);
      }
      if (argval[0] && strcmp(argval,"-")) {
         append_arg(&my_argc, &my_argv, argval);
      }
    }

    fftv_conf_arrN++;
  }

  fclose(fp);



#if 0
  dbg("ooooo\n");
  int i;
  for(i=0;i<my_argc;i++)
    printf("%s ",my_argv[i]);
  dbg("\n");
#endif
  *r_argc = my_argc;
  *r_argv = my_argv;
}


void append_more_args(int *r_argc, char ***r_argv)

{
    append_arg(r_argc, r_argv, orig_output_filename);;
}
