#include <gtk/gtk.h>
#include <sys/types.h>
#include <string.h>
#include <stdio.h>
#include <fftv.h>
#include <vidstr.h>
#include <tv-freq.h>
#include <tv-win.h>

static GtkWidget *label_l, *label_r, *label_b, *label_t;
static GtkWidget *border_adj;


GtkWidget *arrow_button(GtkArrowType type)
{
    GtkWidget *arrow = gtk_arrow_new (type, GTK_SHADOW_IN);
    GtkWidget *button = gtk_button_new();

    gtk_container_add(GTK_CONTAINER(button), arrow);

    return button;
}

extern GtkWidget *tv_fixed_place;
GtkWidget *create_border_adj();
extern GtkWidget *gtv_win;
static int tag_border_timeout;

void close_border_adj()
{
  clr_vidstr(pborder_adj);
  if (border_adj)
    gtk_widget_hide(border_adj);
}

static gboolean timeout_close_border()
{
  close_border_adj();
  tag_border_timeout=0;
  return FALSE;
}

void disp_border_value()
{
  char lstr[16], rstr[16], bstr[16], tstr[16];

  if (!border_adj) {
    border_adj = create_border_adj();
    gtk_fixed_put (GTK_FIXED(tv_fixed_place), border_adj, 0, 0);
  }

  sprintf(lstr,"%d", pchan_current->ofs_x0);
  sprintf(rstr,"%d", pchan_current->ofs_x1);
  sprintf(tstr,"%d", pchan_current->ofs_y0);
  sprintf(bstr,"%d", pchan_current->ofs_y1);

  if (strcmp(gtk_label_get_text(GTK_LABEL(label_l)), lstr))
    gtk_label_set_text(GTK_LABEL(label_l), lstr);
  if (strcmp(gtk_label_get_text(GTK_LABEL(label_r)), rstr))
    gtk_label_set_text(GTK_LABEL(label_r), rstr);
  if (strcmp(gtk_label_get_text(GTK_LABEL(label_b)), bstr))
    gtk_label_set_text(GTK_LABEL(label_b), bstr);
  if (strcmp(gtk_label_get_text(GTK_LABEL(label_t)), tstr))
    gtk_label_set_text(GTK_LABEL(label_t), tstr);

  GtkRequisition sz;
  int x, y;
  gtk_widget_size_request(border_adj, &sz);
  set_vidstr_xy(pborder_adj, sz.width, sz.height,
    x=(win_xl - sz.width) /2, y=(win_yl - sz.height)/2);

//  dbg("%d %d %d %d\n", sz.width, sz.height, x, y);
  gtk_fixed_move (GTK_FIXED(tv_fixed_place), border_adj, pborder_adj->x, pborder_adj->y);

  set_vwin();

  gdk_window_clear_area(gtv_win->window, x, y, sz.width, sz.height);
  gtk_widget_show(border_adj);

  if (tag_border_timeout)
    gtk_timeout_remove(tag_border_timeout);

  tag_border_timeout = gtk_timeout_add(5000, timeout_close_border, NULL);
}


void tv_video_shrink_left(),tv_video_shrink_right(),tv_video_shrink_bottom(),tv_video_shrink_top();
void tv_video_grow_left(),tv_video_grow_right(),tv_video_grow_bottom(),tv_video_grow_top();

static void cb_clicked_ll( GtkWidget *widget, gpointer   data )
{
  printf("ll\n");
  tv_video_shrink_left();
}

static void cb_clicked_lr( GtkWidget *widget, gpointer   data )
{
  printf("lr\n");
  tv_video_grow_left();
}


static void cb_clicked_rl( GtkWidget *widget, gpointer   data )
{
  printf("rl\n");
  tv_video_grow_right();
}


static void cb_clicked_rr( GtkWidget *widget, gpointer   data )
{
  printf("rr\n");
  tv_video_shrink_right();
}


static void cb_clicked_bd( GtkWidget *widget, gpointer   data )
{
  tv_video_shrink_bottom();
  printf("bd\n");
}

static void cb_clicked_bu( GtkWidget *widget, gpointer   data )
{
  tv_video_grow_bottom();
  printf("bu\n");
}


static void cb_clicked_td( GtkWidget *widget, gpointer   data )
{
  tv_video_grow_top();
  printf("td\n");
}


static void cb_clicked_tu( GtkWidget *widget, gpointer   data )
{
  tv_video_shrink_top();
  printf("tu\n");
}


static void cb_clicked_close( GtkWidget *widget, gpointer   data )
{
  printf("close\n");
  close_border_adj();
}


GtkWidget *create_border_adj()
{
    /* Create a 2x2 table */
    GtkWidget *top_frame = gtk_frame_new(NULL);
    GtkWidget *table = gtk_table_new (3, 3, TRUE);

    GtkWidget *hbox_l = gtk_hbox_new (TRUE, 0);
    GtkWidget *arrow_ll = arrow_button (GTK_ARROW_LEFT);
    g_signal_connect (G_OBJECT (arrow_ll), "clicked", G_CALLBACK (cb_clicked_ll), NULL);
    GtkWidget *arrow_lr = arrow_button (GTK_ARROW_RIGHT);
    g_signal_connect (G_OBJECT (arrow_lr), "clicked", G_CALLBACK (cb_clicked_lr), NULL);
    gtk_box_pack_start (GTK_BOX (hbox_l), arrow_ll, TRUE, TRUE, 0);
    label_l = gtk_label_new("l");
    gtk_box_pack_start (GTK_BOX (hbox_l), label_l, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_l), arrow_lr, TRUE, TRUE, 0);
    gtk_table_attach_defaults (GTK_TABLE (table), hbox_l, 0, 1, 1, 2);

    GtkWidget *hbox_r = gtk_hbox_new (TRUE, 0);
    GtkWidget *arrow_rl = arrow_button (GTK_ARROW_LEFT);
    g_signal_connect (G_OBJECT (arrow_rl), "clicked", G_CALLBACK (cb_clicked_rl), NULL);
    GtkWidget *arrow_rr = arrow_button (GTK_ARROW_RIGHT);
    g_signal_connect (G_OBJECT (arrow_rr), "clicked", G_CALLBACK (cb_clicked_rr), NULL);
    gtk_box_pack_start (GTK_BOX (hbox_r), arrow_rl, TRUE, TRUE, 0);
    label_r = gtk_label_new("r");
    gtk_box_pack_start (GTK_BOX (hbox_r), label_r, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_r), arrow_rr, TRUE, TRUE, 0);
    gtk_table_attach_defaults (GTK_TABLE (table), hbox_r, 2, 3, 1, 2);

    GtkWidget *vbox_b = gtk_vbox_new (TRUE, 0);
    GtkWidget *arrow_bd = arrow_button (GTK_ARROW_DOWN);
    g_signal_connect (G_OBJECT (arrow_bd), "clicked", G_CALLBACK (cb_clicked_bd), NULL);
    GtkWidget *arrow_bu = arrow_button (GTK_ARROW_UP);
    g_signal_connect (G_OBJECT (arrow_bu), "clicked", G_CALLBACK (cb_clicked_bu), NULL);
    gtk_box_pack_start (GTK_BOX (vbox_b), arrow_bu, TRUE, TRUE, 0);
    label_b = gtk_label_new("b");
    gtk_box_pack_start (GTK_BOX (vbox_b), label_b, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_b), arrow_bd, TRUE, TRUE, 0);
    gtk_table_attach_defaults (GTK_TABLE (table), vbox_b, 1, 2, 2, 3);

    GtkWidget *vbox_t = gtk_vbox_new (TRUE, 0);
    GtkWidget *arrow_td = arrow_button (GTK_ARROW_DOWN);
    g_signal_connect (G_OBJECT (arrow_td), "clicked", G_CALLBACK (cb_clicked_td), NULL);
    GtkWidget *arrow_tu = arrow_button (GTK_ARROW_UP);
    g_signal_connect (G_OBJECT (arrow_tu), "clicked", G_CALLBACK (cb_clicked_tu), NULL);
    gtk_box_pack_start (GTK_BOX (vbox_t), arrow_tu, TRUE, TRUE, 0);
    label_t = gtk_label_new("t");
    gtk_box_pack_start (GTK_BOX (vbox_t), label_t, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_t), arrow_td, TRUE, TRUE, 0);
    gtk_table_attach_defaults (GTK_TABLE (table), vbox_t, 1, 2, 0, 1);

    GtkWidget *button_ok = gtk_button_new_from_stock(GTK_STOCK_OK);
    g_signal_connect (G_OBJECT (button_ok), "clicked", G_CALLBACK (cb_clicked_close), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), button_ok, 1, 2, 1, 2);

    gtk_widget_show_all(table);

    gtk_container_add(GTK_CONTAINER(top_frame), table);

    return top_frame;
}


void destroy_border_adj()
{
  if (!border_adj)
    return;
  close_border_adj();
  gtk_widget_destroy(border_adj);
  border_adj = NULL;
}
