#include <glib.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "fftv.h"
#include "tv-freq.h"
#include "tv-win.h"

static GtkWidget *tv_desk_opt_window = NULL;
static GtkWidget *opt_tv_win_size, *opt_tv_align_types, *opt_tv_z_order_types;
static GtkWidget *spinner_tv_align_offset_x, *spinner_tv_align_offset_y;
static GtkWidget *check_sticky, *check_decorated, *check_override_redirect;
static char *tv_win_size_str[]={"160x120","240x180","320x240","640x480"};
static int current_tv_win_size_str_idx;
static int tv_win_size_strN = sizeof(tv_win_size_str) / sizeof(tv_win_size_str[0]);
static int current_ofs_x, current_ofs_y;
gboolean b_current_sticky, b_current_decorated, b_override_redirect;

struct {
  char *name;
  TV_WIN_ALIGN_STYLE style;
  char xsign, ysign;
} tv_align_types[] = {
  {"right/top", TV_WIN_ALIGN_RIGHT_TOP, '-', '+'},
  {"right/bottom", TV_WIN_ALIGN_RIGHT_BOTTOM, '-', '-'},
  {"left/top", TV_WIN_ALIGN_LEFT_TOP, '+', '+'},
  {"left/bottom", TV_WIN_ALIGN_LEFT_BOTTOM, '+', '-'},
  {"None", TV_WIN_ALIGN_NONE, ' ', ' '},
};
int current_tv_align_types_idx;
int tv_align_typesN=sizeof(tv_align_types)/sizeof(tv_align_types[0]);


struct {
  char *name;
  GdkWindowTypeHint gtypehint;
} tv_z_order_types[] = {
  { "Stay-on-Top", GDK_WINDOW_TYPE_HINT_SPLASHSCREEN},
  { "Background", GDK_WINDOW_TYPE_HINT_DESKTOP},
  { "Normal", GDK_WINDOW_TYPE_HINT_NORMAL},
};

static int tv_z_order_typesN = sizeof(tv_z_order_types) / sizeof(tv_z_order_types[0]);
static int current_tv_z_order_types_idx;
GdkWindowTypeHint current_GDK_WINDOW_TYPE_HINT = GDK_WINDOW_TYPE_HINT_NORMAL;

static void close_tv_desk_opt_window()
{
  gtk_widget_destroy(tv_desk_opt_window);
  tv_desk_opt_window = NULL;
}


static char *desk_opt_file_name()
{
  static char filename[256];

  get_fftv_states_dir(filename);
  strcat(filename,"/tv-desktop-option");

  return filename;
}

static void save_desk_opt()
{
  char *file = desk_opt_file_name();

  FILE *fp;

  if ((fp=fopen(file, "w"))==NULL) {
    p_err("cannot write to file %s\n", file);
  }

  fprintf(fp,"win_size %s\n", tv_win_size_str[current_tv_win_size_str_idx]);
  fprintf(fp,"align_offset %d %d\n", current_ofs_x, current_ofs_y);
  fprintf(fp,"z-order %s\n", tv_z_order_types[current_tv_z_order_types_idx].name);
  fprintf(fp,"sticky %d\n", b_current_sticky);
  fprintf(fp,"decorated %d\n", b_current_decorated);
  fprintf(fp,"ovride-redirect %d\n", b_override_redirect);
  fclose(fp);
}

#if 0
static void set_tv_align_types_by_name(char *name)
{
  int i;

  dbg("set_tv_align_types_by_name %s\n", name);

  for(i=0; i< tv_align_typesN; i++)
    if (!strcmp(tv_align_types[i].name, name)) {
      current_tv_align_types_idx = i;
      dbg("current_tv_align_types_idx: %d %s", i, name);
      return;
    }
}
#endif


static void set_tv_z_order_types_by_name(char *name)
{
  int i;

  for(i=0; i< tv_z_order_typesN; i++)
    if (!strcmp(tv_z_order_types[i].name, name)) {
      current_tv_z_order_types_idx = i;
      return;
    }
}


static void set_tv_win_size_str_by_name(char *name)
{
  int i;

  for(i=0; i< tv_win_size_strN; i++)
    if (!strcmp(tv_win_size_str[i], name)) {
      current_tv_win_size_str_idx = i;

      return;
    }
}



void load_desk_opt()
{
  char *file = desk_opt_file_name();

  FILE *fp;

  if ((fp=fopen(file, "r"))==NULL) {
    return;
  }

  dbg("load_desk_opt: %s\n", file);

  char aa[128], bb[128];

  fscanf(fp,"%s %s", aa, bb);
  set_tv_win_size_str_by_name(bb);
  fscanf(fp,"%s %d %d", aa, &current_ofs_x, &current_ofs_y);
  fscanf(fp,"%s %s", aa, bb);
  set_tv_z_order_types_by_name(bb);
  fscanf(fp,"%s %d", aa, &b_current_sticky);
  fscanf(fp,"%s %d", aa, &b_current_decorated);
  fscanf(fp,"%s %d", aa, &b_override_redirect);

  fclose(fp);
}


static gboolean close_application( GtkWidget *widget,
                                   GdkEvent  *event,
                                   gpointer   data )
{
  close_tv_desk_opt_window();
  return FALSE;
}


static gboolean cb_cancel( GtkWidget *widget,
                                   GdkEvent  *event,
                                   gpointer   data )
{
  close_tv_desk_opt_window();
  return FALSE;
}

void fftv_grab_close();

extern char *geomstr;
void create_tv_win(gboolean, gboolean);

static gboolean cb_ok( GtkWidget *widget, GdkEvent  *event, gpointer   data )
{
  int align_style_idx = gtk_option_menu_get_history(GTK_OPTION_MENU(opt_tv_align_types));
  current_tv_align_types_idx = align_style_idx;

  static char ngeomstr[64];
  char xsign, ysign;
  int ofs_x = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinner_tv_align_offset_x));
  int ofs_y = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinner_tv_align_offset_y));
  current_tv_win_size_str_idx = gtk_option_menu_get_history(GTK_OPTION_MENU(opt_tv_win_size));
  char *size_str = tv_win_size_str[current_tv_win_size_str_idx];

  xsign = tv_align_types[align_style_idx].xsign;
  ysign = tv_align_types[align_style_idx].ysign;

  current_ofs_x = ofs_x;
  current_ofs_y = ofs_y;

  current_tv_z_order_types_idx = gtk_option_menu_get_history(GTK_OPTION_MENU(opt_tv_z_order_types));
  b_current_sticky = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_sticky));
  b_current_decorated = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_decorated));
  b_override_redirect = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_override_redirect));
  if (b_override_redirect)
    b_current_decorated = FALSE;

  save_desk_opt();

  if (xsign==' ')
    sprintf(ngeomstr, "%s", size_str);
  else
    sprintf(ngeomstr, "%s%c%d%c%d", size_str, xsign, ofs_x, ysign, ofs_y);

  dbg("-- %s   %s,%d\n", ngeomstr, tv_z_order_types[current_tv_z_order_types_idx].name,
        b_current_sticky);

  current_GDK_WINDOW_TYPE_HINT = tv_z_order_types[current_tv_z_order_types_idx].gtypehint;

  geomstr = ngeomstr;

  if (!TV_WIN_DISP_MODE_GRAB)
    tv_pic_off();

  create_tv_win(TRUE, TRUE);

  close_tv_desk_opt_window();

  break_tv_disp_loop(FALSE);

  return FALSE;
}


extern GtkWidget *gtv_win;


void cb_opt_tv_win_size_menu_select(GtkWidget  *item,  GtkWidget *opt)
{
   int idx = gtk_option_menu_get_history(GTK_OPTION_MENU(opt));

   printf("select %s\n", tv_win_size_str[idx]);
}


void cb_opt_align_types_menu_select(GtkWidget  *item,  GtkWidget *opt)
{
#if 0
   int idx = gtk_option_menu_get_history(GTK_OPTION_MENU(opt));

   printf("select %s\n", tv_align_types[idx].name);
#endif
}

void cb_spinner_value_changed_align_offset_x(GtkSpinButton *spinbutton,
                                      gpointer user_data)
{
#if 0
  int value = (int) gtk_spin_button_get_value(spinbutton);

  dbg("x:%d\n", value);
#endif
}


void cb_spinner_value_changed_align_offset_y(GtkSpinButton *spinbutton,
                                      gpointer user_data)
{
#if 0
  int value = (int) gtk_spin_button_get_value(spinbutton);

  dbg("y:%d\n", value);
#endif
}

void cb_opt_tv_win_z_order_menu_select(GtkWidget  *item,  GtkWidget *opt)
{
   int idx = gtk_option_menu_get_history(GTK_OPTION_MENU(opt));

   printf("select %s\n", tv_z_order_types[idx].name);
}


void align_with_ui_window(GtkWidget *win);

void create_tv_desktop_option_window()
{
    GtkWidget *box_top;

    if (tv_desk_opt_window) {
      gtk_window_present(GTK_WINDOW(tv_desk_opt_window));
      return;
    }

    load_desk_opt();

    tv_desk_opt_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    g_signal_connect (G_OBJECT (tv_desk_opt_window), "delete_event",
                      G_CALLBACK (close_application),
                      NULL);

    gtk_window_set_title (GTK_WINDOW (tv_desk_opt_window), "TV Desktop Options");
    gtk_container_set_border_width (GTK_CONTAINER (tv_desk_opt_window), 3);
    align_with_ui_window(tv_desk_opt_window);


    box_top = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (tv_desk_opt_window), box_top);


    GtkWidget *frame_tv_win = gtk_frame_new("Window geometry");
    gtk_box_pack_start (GTK_BOX (box_top), frame_tv_win, FALSE, FALSE, 0);
    GtkWidget *vbox_frame_tv_win = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (frame_tv_win), vbox_frame_tv_win);
    gtk_container_set_border_width (GTK_CONTAINER (vbox_frame_tv_win), 5);

    GtkWidget *hbox_tv_win = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_frame_tv_win), hbox_tv_win, FALSE, FALSE, 0);


    GtkWidget *vbox_opt_tv_win_size = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_tv_win), vbox_opt_tv_win_size, FALSE, FALSE, 0);
    GtkWidget *label_tv_win_size = gtk_label_new("Size");
    gtk_box_pack_start (GTK_BOX (vbox_opt_tv_win_size), label_tv_win_size, FALSE, FALSE, 0);

    opt_tv_win_size = gtk_option_menu_new ();
    GtkWidget *menu_tv_win_size = gtk_menu_new ();
    dbg("tv_win_size_strN:%d\n", tv_win_size_strN);

    int i;

    for(i=0; i < tv_win_size_strN; i++) {
      GtkWidget *item = gtk_menu_item_new_with_label (tv_win_size_str[i]);
      g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK(cb_opt_tv_win_size_menu_select),
                        opt_tv_win_size);
      gtk_menu_shell_append (GTK_MENU_SHELL (menu_tv_win_size), item);
    }
    gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_tv_win_size), menu_tv_win_size);
    gtk_box_pack_start (GTK_BOX (vbox_opt_tv_win_size), opt_tv_win_size, FALSE, FALSE, 3);
    gtk_option_menu_set_history (GTK_OPTION_MENU (opt_tv_win_size), current_tv_win_size_str_idx);

    GtkWidget *vbox_opt_tv_align_types = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_tv_win), vbox_opt_tv_align_types, FALSE, FALSE, 5);
    GtkWidget *label_tv_align_types = gtk_label_new("Desktop Align Type");
    gtk_box_pack_start (GTK_BOX (vbox_opt_tv_align_types), label_tv_align_types, FALSE, FALSE, 0);

    opt_tv_align_types = gtk_option_menu_new ();
    GtkWidget *menu_tv_align_types = gtk_menu_new ();
    int tv_align_typesN = sizeof(tv_align_types) / sizeof(tv_align_types[0]);
    dbg("tv_align_typesN:%d\n", tv_align_typesN);

    for(i=0; i < tv_align_typesN; i++) {
      GtkWidget *item = gtk_menu_item_new_with_label (tv_align_types[i].name);
      g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK(cb_opt_align_types_menu_select),
                        opt_tv_align_types);
      gtk_menu_shell_append (GTK_MENU_SHELL (menu_tv_align_types), item);
    }
    gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_tv_align_types), menu_tv_align_types);
    gtk_box_pack_start (GTK_BOX (vbox_opt_tv_align_types), opt_tv_align_types, FALSE, FALSE, 3);
    gtk_option_menu_set_history (GTK_OPTION_MENU (opt_tv_align_types), current_tv_align_types_idx);


    GtkWidget *hbox_tv_align_offset = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_frame_tv_win), hbox_tv_align_offset, FALSE, FALSE, 5);

    GtkWidget *vbox_tv_align_offset_x = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_tv_align_offset), vbox_tv_align_offset_x, FALSE, FALSE, 0);
    GtkWidget *label_tv_align_offset_x = gtk_label_new("Align offset X");
    gtk_box_pack_start (GTK_BOX (vbox_tv_align_offset_x), label_tv_align_offset_x, FALSE, FALSE, 0);

    double step = 1.0;
    GtkAdjustment *adj_tv_align_offset_x = (GtkAdjustment *) gtk_adjustment_new (
         current_ofs_x, 0.0, 640.0, step, step, step);
    spinner_tv_align_offset_x = gtk_spin_button_new (adj_tv_align_offset_x, step, 0);

    g_signal_connect (G_OBJECT(spinner_tv_align_offset_x), "value-changed",
                      G_CALLBACK(cb_spinner_value_changed_align_offset_x), NULL);

    gtk_widget_set_size_request (spinner_tv_align_offset_x, 60, -1);
    gtk_box_pack_start (GTK_BOX (vbox_tv_align_offset_x), spinner_tv_align_offset_x, FALSE, FALSE, 0);


    GtkWidget *vbox_tv_align_offset_y = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_tv_align_offset), vbox_tv_align_offset_y, FALSE, FALSE, 15);
    GtkWidget *label_tv_align_offset_y = gtk_label_new("Align offset Y");
    gtk_box_pack_start (GTK_BOX (vbox_tv_align_offset_y), label_tv_align_offset_y, FALSE, FALSE, 0);

    GtkAdjustment *adj_tv_align_offset_y = (GtkAdjustment *) gtk_adjustment_new (
         current_ofs_y, 0.0, 640.0, step, step, step);
    spinner_tv_align_offset_y = gtk_spin_button_new (adj_tv_align_offset_y, step, 0);

    g_signal_connect (G_OBJECT(spinner_tv_align_offset_y), "value-changed",
                      G_CALLBACK(cb_spinner_value_changed_align_offset_y), NULL);

    gtk_widget_set_size_request (spinner_tv_align_offset_y, 60, -1);

    gtk_box_pack_start (GTK_BOX (vbox_tv_align_offset_y), spinner_tv_align_offset_y, FALSE, FALSE, 0);


    GtkWidget *hbox_desk_check = gtk_hbox_new (TRUE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_frame_tv_win), hbox_desk_check, FALSE, FALSE, 5);

    GtkWidget *vbox_desk_option_z_order = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_desk_check), vbox_desk_option_z_order, FALSE, FALSE, 0);
    GtkWidget *label_desk_option_z_order = gtk_label_new("Z order");
    gtk_box_pack_start (GTK_BOX (vbox_desk_option_z_order), label_desk_option_z_order,
                        FALSE, FALSE, 0);

    opt_tv_z_order_types = gtk_option_menu_new ();
    GtkWidget *menu_tv_z_order_types = gtk_menu_new ();
    dbg("tv_z_order_typesN:%d\n", tv_z_order_typesN);

    for(i=0; i < tv_z_order_typesN; i++) {
      GtkWidget *item = gtk_menu_item_new_with_label (tv_z_order_types[i].name);
      g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK(cb_opt_tv_win_z_order_menu_select),
                        opt_tv_z_order_types);
      gtk_menu_shell_append (GTK_MENU_SHELL (menu_tv_z_order_types), item);
    }
    gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_tv_z_order_types), menu_tv_z_order_types);
    gtk_box_pack_start (GTK_BOX (vbox_desk_option_z_order), opt_tv_z_order_types, FALSE, FALSE, 3);
    gtk_option_menu_set_history (GTK_OPTION_MENU (opt_tv_z_order_types), current_tv_z_order_types_idx);


    GtkWidget *vbox_desk_check_sticky = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_desk_check), vbox_desk_check_sticky, FALSE, FALSE, 0);
    GtkWidget *label_desk_check_sticky = gtk_label_new("Sticky");
    gtk_box_pack_start (GTK_BOX (vbox_desk_check_sticky), label_desk_check_sticky,
                        FALSE, FALSE, 0);
    check_sticky = gtk_check_button_new ();
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_sticky), b_current_sticky);
    gtk_box_pack_start (GTK_BOX (vbox_desk_check_sticky), check_sticky,
                        FALSE, FALSE, 0);

    GtkWidget *hbox_desk_check2 = gtk_hbox_new (TRUE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_frame_tv_win), hbox_desk_check2, FALSE, FALSE, 5);

    GtkWidget *vbox_desk_check_decorated = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_desk_check2), vbox_desk_check_decorated, FALSE, FALSE, 0);
    GtkWidget *label_desk_check_decorated = gtk_label_new("Decorated");
    gtk_box_pack_start (GTK_BOX (vbox_desk_check_decorated), label_desk_check_decorated,
                        FALSE, FALSE, 0);
    check_decorated = gtk_check_button_new ();
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_decorated), b_current_decorated);
    gtk_box_pack_start (GTK_BOX (vbox_desk_check_decorated), check_decorated,
                        FALSE, FALSE, 0);

    GtkWidget *vbox_desk_check_override_redirect = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_desk_check2), vbox_desk_check_override_redirect, FALSE, FALSE, 0);
    GtkWidget *label_desk_check_override_redirect = gtk_label_new("Override redirect");
    gtk_box_pack_start (GTK_BOX (vbox_desk_check_override_redirect), label_desk_check_override_redirect,
                        FALSE, FALSE, 0);
    check_override_redirect = gtk_check_button_new ();
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_override_redirect), b_override_redirect);
    gtk_box_pack_start (GTK_BOX (vbox_desk_check_override_redirect), check_override_redirect,
                        FALSE, FALSE, 0);


    GtkWidget *hbox_ok_cancel = gtk_hbox_new (TRUE, 0);
    gtk_box_pack_start (GTK_BOX (box_top), hbox_ok_cancel, FALSE, FALSE, 5);

    GtkWidget *button_cancel =gtk_button_new_from_stock (GTK_STOCK_CANCEL);
    g_signal_connect_swapped (G_OBJECT (button_cancel), "clicked",
                              G_CALLBACK (cb_cancel),
                              G_OBJECT (tv_desk_opt_window));
    gtk_box_pack_start (GTK_BOX (hbox_ok_cancel), button_cancel, TRUE, TRUE, 10);

    GtkWidget *button_ok = gtk_button_new_from_stock (GTK_STOCK_OK);
    g_signal_connect_swapped (G_OBJECT (button_ok), "clicked",
                              G_CALLBACK (cb_ok),
                              G_OBJECT (tv_desk_opt_window));
    gtk_box_pack_start (GTK_BOX (hbox_ok_cancel), button_ok, TRUE, TRUE, 10);

    GTK_WIDGET_SET_FLAGS (button_ok, GTK_CAN_DEFAULT);

    gtk_widget_grab_default (button_ok);

    gtk_widget_show_all (tv_desk_opt_window);
}
