#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include "gtk/gtk.h"
#include "tv-freq.h"
#include "fftv.h"
#include "vidstr.h"

static char *av_player;
gboolean g_LinePlaybackAutoSwitch=TRUE;

static char *get_player_exec()
{
  static char player_exec[256];
  char fftv_states_dir[256];

  get_fftv_states_dir(fftv_states_dir);

  sprintf(player_exec, "%s/tv-audio-video-player", fftv_states_dir);
  return player_exec;
}


void load_av_player_conf()
{

  if (av_player)
    return;

  char *player_exec = get_player_exec();

  struct {
    char *name;
    char *arg;
  } players[]={
    {"xine"},
    {"gxine"},
    {"totem"},
    {"mplayer", "-quiet" },
    {"gmplayer"},
  };
  static int playersN = sizeof(players) / sizeof(players[0]);

  FILE *fp;
  if ((fp=fopen(player_exec,"r"))==NULL) {
    char tt[256];

    strcpy(tt, players[0].name);

    int i;
    for(i=0; i < playersN; i++) {
      sprintf(tt, "which %s >& /dev/null", players[i].name);

      int status;

      if ((status = system(tt))) {
        dbg("%s status %d\n", tt, status);
        continue;
      }

      if (players[i].arg)
        sprintf(tt, "%s %s", players[i].name, players[i].arg);
      else
        strcpy(tt, players[i].name);

      dbg("player will use %s\n", tt);
      break;
    }

    av_player = strdup(tt);
    return;
  }

  char buf[256];

  buf[0]=0;
  fgets(buf,sizeof(buf),fp);

  if (buf[strlen(buf)-1]=='\n')
      buf[strlen(buf)-1]=0;

  av_player = strdup(buf);

  char tstr[256];

  fscanf(fp, "%s %d", tstr, &g_LinePlaybackAutoSwitch);

  fclose(fp);
}


static GtkWidget *player_select_window;
static GtkWidget *entry_player;

void close_player_select_window()
{
  gtk_widget_destroy(player_select_window);
}

static void cb_cancel()
{
  close_player_select_window();
}


static void cb_ok()
{
  const char *text_player = gtk_entry_get_text(GTK_ENTRY(entry_player));

  av_player = strdup(text_player);

  char *player_exec = get_player_exec();

  FILE *fp;
  if ((fp=fopen(player_exec,"w"))) {
    fprintf(fp,"%s\n", av_player);
    fprintf(fp,"LinePlaybackAutoSwitch %d\n", g_LinePlaybackAutoSwitch);
    fclose(fp);
  }

  close_player_select_window();
}


void cb_check_button_toggled_lineauto(GtkToggleButton *togglebutton,
                                      gpointer user_data)
{
  gboolean val = gtk_toggle_button_get_active(togglebutton);
  dbg("val:%d\n",val);

  g_LinePlaybackAutoSwitch = val;
}


void create_player_options_window()
{
    GtkWidget *hbox_file;
    GtkWidget *label_file;

    load_av_player_conf();

    player_select_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (player_select_window), 10);

    g_signal_connect (G_OBJECT (player_select_window), "delete_event",
                        G_CALLBACK (cb_cancel), NULL);

    gtk_window_set_title (GTK_WINDOW (player_select_window), "Enter Audio/Video av_player");

    GtkWidget *vbox = gtk_vbox_new (FALSE, 5);

    hbox_file = gtk_hbox_new (FALSE, 0);

    label_file = gtk_label_new("Audio/Video av_player");
    gtk_box_pack_start (GTK_BOX (hbox_file), label_file, FALSE, TRUE, 0);

    entry_player = gtk_entry_new ();

#if 0
    g_signal_connect (G_OBJECT (entry), "activate",
		      G_CALLBACK (enter_callback),
		      (gpointer) entry);
#endif
    gtk_entry_set_max_length (GTK_ENTRY (entry_player), 128);
    gtk_entry_set_width_chars(GTK_ENTRY (entry_player), 15);
    gtk_entry_set_text (GTK_ENTRY (entry_player), av_player);
//    gtk_widget_set_size_request(entry, 100,24);
    gtk_box_pack_start (GTK_BOX (hbox_file), entry_player, FALSE, TRUE, 10);

    gtk_box_pack_start (GTK_BOX (vbox), hbox_file, TRUE, TRUE, 0);

    GtkWidget *separator_file = gtk_hseparator_new ();

    gtk_box_pack_start (GTK_BOX (vbox), separator_file, TRUE, TRUE, 3);

    GtkWidget *hbox_lineauto = gtk_hbox_new (FALSE, 2);

    GtkWidget *check_button_lineauto = gtk_check_button_new_with_label
                                       ("Line Playback AutoSwitch");
    g_signal_connect (G_OBJECT (check_button_lineauto), "toggled",
                      G_CALLBACK (cb_check_button_toggled_lineauto), NULL);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_lineauto),
                      g_LinePlaybackAutoSwitch);

    gtk_box_pack_start (GTK_BOX (hbox_lineauto), check_button_lineauto, TRUE, TRUE, 2);

    gtk_box_pack_start (GTK_BOX (vbox), hbox_lineauto, TRUE, TRUE, 2);

    GtkWidget *hbox_ok = gtk_hbox_new (TRUE, 0);

    GtkWidget *button_cancel =gtk_button_new_from_stock (GTK_STOCK_CANCEL);
    gtk_box_pack_start (GTK_BOX (hbox_ok), button_cancel, TRUE, TRUE, 10);
    g_signal_connect (GTK_OBJECT (button_cancel), "clicked", G_CALLBACK (cb_cancel), NULL);

    GtkWidget *button_ok = gtk_button_new_from_stock (GTK_STOCK_OK);
    gtk_box_pack_start (GTK_BOX (hbox_ok), button_ok, TRUE, TRUE, 10);
    g_signal_connect (GTK_OBJECT (button_ok), "clicked", G_CALLBACK (cb_ok), NULL);

    gtk_box_pack_start (GTK_BOX (vbox), hbox_ok, TRUE, TRUE, 10);

    gtk_container_add (GTK_CONTAINER (player_select_window), vbox);

    gtk_widget_show_all (player_select_window);
}

// set the speaker output of line input, but not recording
void set_LinePlaybackSwitch(int val)
{
  char *onoff=val ? "on,on":"off,off";
  char cmdstr[256];
  int rstatus;

  sprintf(cmdstr,"amixer cset iface=MIXER,name='Line Playback Switch' %s >& /dev/null", onoff);
  if ((rstatus=system(cmdstr)) < 0) {
    dbg("***** ERROR:   cannot execute '%s'  status:%d\n", cmdstr, rstatus);
    dbg("** ALSA utils was not installed\n");
    dbg("** If you use Mandrake 9.2, please install alsa-utils-0.9.6-3mdk\n m");
  }
}


#include <signal.h>


extern char output_filename_actual[];

void play_current_file()
{
  char cmd[1024];
  int pid;

  load_av_player_conf();

  signal(SIGCHLD,SIG_IGN);

  pid=fork();

  if (pid > 0)
    return;
  if (pid < 0)
    p_err("cannot fork a process");

  signal(SIGCHLD,SIG_DFL);

  if (g_LinePlaybackAutoSwitch)
    set_LinePlaybackSwitch(0);

  snprintf(cmd, sizeof(cmd), "%s %s", av_player, output_filename_actual);
  int status;

  dbg("exec player cmd %s\n", cmd);

  if ((status=system(cmd)) !=0)
    dbg("cannot execute '%s', exit with status %d\n", cmd, status);

  if (g_LinePlaybackAutoSwitch)
    set_LinePlaybackSwitch(1);

  exit(0);
}
