#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include <linux/videodev.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <time.h>
#include <pthread.h>
#include <linux/soundcard.h>

#if 0
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>
#include <X11/keysym.h>
#include <X11/cursorfont.h>
#include <X11/Xproto.h>
#include <X11/Xatom.h>
#include <X11/XF86keysym.h>
#endif

#include <string.h>
#include <ctype.h>
#include <gtk/gtk.h>
#include "tv-freq.h"
#include "fftv.h"

static int mixer_vol;
int mixer_fd;

#define MIXER_DEV "/dev/mixer"
#define DSP_DEV "/dev/dsp"
int dsp_fd=0;

void soundInit(int fd)
{
#if !MTV
    int fmt=AFMT_U8;
    int frag=0x7fff000c; // 4k
    int channels=1;
//    int rate=44100;
    int rate=8000;
    int trigger;
    int prof=APF_NORMAL;
    int samplesize=8;
    int dsp_stereo=0;
    int blksize;
    struct stat st;
#endif

  if (fd)
    dsp_fd=fd;
  else {
    if (dsp_fd)
      close(dsp_fd);
    if ((dsp_fd = open(DSP_DEV,O_RDWR)) < 0 ) {
      perror("open " DSP_DEV);
      return;
    }
  }

#if !MTV

//    fcntl(fd,F_SETFD,FD_CLOEXEC);

    ioctl(dsp_fd, SNDCTL_DSP_SETFMT, &fmt);
    ioctl(dsp_fd, SNDCTL_DSP_PROFILE, &prof);
    ioctl(dsp_fd, SNDCTL_DSP_STEREO, &dsp_stereo);
    ioctl(dsp_fd, SNDCTL_DSP_SPEED, &rate);
    ioctl(dsp_fd, SNDCTL_DSP_CHANNELS, &channels);
    ioctl(dsp_fd, SNDCTL_DSP_SETFRAGMENT, &frag);
    ioctl(dsp_fd, SNDCTL_DSP_SAMPLESIZE, &samplesize);
    ioctl(dsp_fd, SNDCTL_DSP_GETBLKSIZE, &blksize);

    trigger=0;
    ioctl(dsp_fd,SNDCTL_DSP_SETTRIGGER,&trigger);

    // ALSA doesn't need this
    if (stat("/dev/snd",&st)) {  //  Why do we need this ?
#if 1
         int abuf_size;
         char *audiobuf;

         ioctl (dsp_fd, SNDCTL_DSP_GETBLKSIZE, &abuf_size);

         if ((audiobuf = malloc (abuf_size)) == NULL) {
           fprintf (stderr, "Unable to allocate input/output buffer\n");
           exit (-1);
         }

         read (dsp_fd, audiobuf, abuf_size);

         free(audiobuf);
#else
	int oflags,flags,rc;
	unsigned char buf[4096];

	oflags = fcntl(dsp_fd,F_GETFL);
	flags = oflags | O_NONBLOCK;
	fcntl(dsp_fd,F_SETFL,flags);
	for (;;) {
	  rc = read(dsp_fd,buf,sizeof(buf));
	  if (rc != sizeof(buf))
	    break;
	}
	fcntl(dsp_fd,F_SETFL,oflags);
#endif
    }
    trigger=PCM_ENABLE_INPUT;
    ioctl(dsp_fd,SNDCTL_DSP_SETTRIGGER,&trigger);

#endif
    if (!fd) {
      close(dsp_fd);
      dsp_fd=0;
    }

//  dbg("%d  %d %d\n",mixer_vol, mixer_vol>>8, mixer_vol&0xff);
}

int get_soundcard_volume()
{
  int vol;

  if (ioctl(mixer_fd, MIXER_READ(SOUND_MIXER_VOLUME), &vol) < 0 ) {
    perror("read vol 2");
    return -1;
  }

  return vol & 0xff;
}


void mixerInit()
{
  if (mixer_fd)
    return;

  if ((mixer_fd = open(MIXER_DEV,O_RDWR)) < 0 ) {
    perror("open " MIXER_DEV);
    return;
  }

  mixer_vol = get_soundcard_volume();
}


void mixerClose()
{
  if (!mixer_fd)
    return;

  close(mixer_fd);
  mixer_fd=0;
}

int get_soundcard_volume();
void update_radio_hscale();

static int orig_vol;

void soundcard_mute_toggle()
{
  int nvol;

  if (orig_vol) {
    mixer_vol = orig_vol;
    orig_vol = 0;
    nvol=(mixer_vol<<8)|mixer_vol;

    if (fftv_mode & FFTV_MODE_TV)
      draw_vol_off();
  } else {
    orig_vol = get_soundcard_volume();
    mixer_vol = nvol=0;

    if (fftv_mode & FFTV_MODE_TV)
      draw_vol_status("Mute",0);

  }


  if (ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_VOLUME), &nvol) < 0 ) {
    perror("write vol");
    goto ret;
  }


  if (fftv_mode & FFTV_MODE_RADIO) {
    update_radio_hscale();
  }

ret:
  return;
}


void line1_record_on()
{
  system("aumix -l R");
}


#define STEP 2


void update_tv_hscale();


void soundcard_vol_set(int vol, int b_draw_vol_status)
{
  mixer_vol = vol;

  int vv = (vol << 8) | vol;

//  mixerInit();

  if (ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_VOLUME),&vv) < 0 ) {
    perror("write vol");
    return;
  }
//  mixerClose();

  if (fftv_mode & FFTV_MODE_RADIO) {
    update_radio_hscale();
  } else {
    char s[4];

    sprintf(s,"%d",vol);

    if (b_draw_vol_status)
      draw_vol_status(s,1);

    update_tv_hscale();
    save_channel_state("volume",vol);
  }
}


// some desktops has grabbed the X volume up/down keys, use this to fetch it
gint soundcard_vol_fetch_update()
{
  int current_vol = get_soundcard_volume();

  if (current_vol == mixer_vol)
    return TRUE;

  soundcard_vol_set(current_vol, TRUE);

  return TRUE;
}


void soundcard_vol_adj(int sign)
{
  int vol;
  int current_vol = get_soundcard_volume();

  if (current_vol==0 && orig_vol)
    vol = orig_vol;
  else
    vol = current_vol;

  vol += STEP*sign;

  if (vol < 0)
    vol = 0;
  if (vol > 100)
    vol = 100;

  soundcard_vol_set(vol, TRUE);
}

void soundcard_raise_vol()
{
  soundcard_vol_adj(1);
}

void soundcard_lower_vol()
{
  soundcard_vol_adj(-1);
}


void soundcard_raise_vol4()
{
  soundcard_vol_adj(4);
}

void soundcard_lower_vol4()
{
  soundcard_vol_adj(-4);
}

void set_soundcard_volume(int vol)
{
//  dbg("set_soundcard_volume %d\n", vol);
  soundcard_vol_set(vol, FALSE);
}
