#include <glib.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include <sys/time.h>
#include <linux/videodev.h>
#include <string.h>
#include "tv-freq.h"

static GtkWidget *tv_std_window = NULL;

static int new_select_idx;

TV_STANDARD tv_std_arr[] = {
  {"NTSC", VIDEO_MODE_NTSC, V4L2_STD_NTSC },
  {"NTSCjp", VIDEO_MODE_NTSC, V4L2_STD_NTSC_M_JP },
  {"PAL", VIDEO_MODE_PAL, V4L2_STD_PAL},
  {"PAL_M", VIDEO_MODE_PAL, V4L2_STD_PAL_M},
  {"PAL_N", VIDEO_MODE_PAL, V4L2_STD_PAL_N},
  {"PAL_Nc", VIDEO_MODE_PAL, V4L2_STD_PAL_Nc},
  {"SECAM", VIDEO_MODE_SECAM, V4L2_STD_SECAM},
  {"AUTO", VIDEO_MODE_AUTO, V4L2_STD_UNKNOWN}
};

static int tv_std_arrN = sizeof(tv_std_arr) / sizeof(tv_std_arr[0]);
void set_tv_standard(TV_STANDARD *ptv_standard);
TV_STANDARD *pcurrent_tv_std;

void set_tv_standard_by_name(char *name)
{
  int i;

//  dbg("set_tv_standard_by_name %s\n", name);

  for(i=0;i < tv_std_arrN; i++)
    if (!strcmp(tv_std_arr[i].name, name)) {
      pcurrent_tv_std = &tv_std_arr[i];
      set_tv_standard(pcurrent_tv_std);
      return;
    }

  p_err("set_tv_standard_by_name: %s is not valid\n", name);
}


static char *tv_standard_file_name()
{
  static char filename[256];

  get_fftv_states_dir(filename);
  strcat(filename,"/tv-standard");

  return filename;
}

static void save_tv_standard()
{
  char *file = tv_standard_file_name();

  FILE *fp;

  if ((fp=fopen(file, "w"))==NULL) {
    p_err("cannot write to file %s\n", file);
  }

  fprintf(fp,"%s\n", pcurrent_tv_std->name);
  fclose(fp);
}

void load_tv_standard()
{
  char *file = tv_standard_file_name();

  FILE *fp;

  if ((fp=fopen(file, "r"))==NULL) {
    return;
  }

  dbg("load_tv_standard: %s\n", file);

  char aa[128];

  fscanf(fp,"%s", aa);

  fclose(fp);

  set_tv_standard_by_name(aa);
}



static gboolean close_application( GtkWidget *widget,
                                   GdkEvent  *event,
                                   gpointer   data )
{
  gtk_widget_destroy(tv_std_window);

  return FALSE;
}



static gboolean cb_ok( GtkWidget *widget,
                                   GdkEvent  *event,
                                   gpointer   data )
{
  printf("--- close with %d\n", new_select_idx);
  set_tv_standard_by_name(tv_std_arr[new_select_idx].name);
  save_tv_standard();
  gtk_widget_destroy(tv_std_window);
  return FALSE;
}


static void callback_button_clicked( GtkWidget *widget, gpointer data)
{
//  printf("selected %d\n", data);
  new_select_idx = (int) data;
}

extern GtkWidget *gtv_win;

void create_tv_std_window()
{
    GtkWidget *box1;
    GtkWidget *box2;
    GtkWidget *button, *button_close;
    GtkWidget *separator;
    GtkWidget *frame;

    load_tv_standard();

    tv_std_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    align_with_ui_window(tv_std_window);

    g_signal_connect (G_OBJECT (tv_std_window), "delete_event",
                      G_CALLBACK (close_application),
                      NULL);

    gtk_window_set_title (GTK_WINDOW (tv_std_window), "TV standard");
    gtk_container_set_border_width (GTK_CONTAINER (tv_std_window), 3);

    frame = gtk_frame_new("Select a TV standard");
    gtk_container_add (GTK_CONTAINER (tv_std_window), frame);


    box1 = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (frame), box1);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);

    int i;
    GSList *group=NULL;

    int current_idx = 0;

    if (current_idx == chanlistsN)
      current_idx = 0;

    new_select_idx = current_idx;

    for(i=0; i<tv_std_arrN; i++) {
      button = gtk_radio_button_new_with_label (group, tv_std_arr[i].name);
      gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);

      group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));

      g_signal_connect (G_OBJECT (button), "clicked",
         G_CALLBACK (callback_button_clicked), (gpointer) i);

      if (&tv_std_arr[i]==pcurrent_tv_std)
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
    }

    separator = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);

    button_close = gtk_button_new_from_stock (GTK_STOCK_OK);
    g_signal_connect_swapped (G_OBJECT (button_close), "clicked",
                              G_CALLBACK (cb_ok),
                              G_OBJECT (tv_std_window));
    gtk_box_pack_start (GTK_BOX (box2), button_close, TRUE, TRUE, 0);
    GTK_WIDGET_SET_FLAGS (button_close, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (button_close);

    gtk_widget_show_all (tv_std_window);

    return;
}
