#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/vfs.h>
#define _I386_STATFS_H   // to skip /usr/include/asm/statfs.h on Mandrake 9.0

#include <linux/videodev.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>
#include <sys/shm.h>
#include <linux/soundcard.h>

#include <string.h>
#include <ctype.h>


#include "tv-freq.h"
#include "fftv.h"
#include "tv-win.h"

static int zero=0,one=1;
int b_v4l2;

extern struct video_picture vpict;

void get_tv_picture_ctrl()
{
  if (b_v4l2) {
    struct v4l2_control ctrl;

    ctrl.id=V4L2_CID_CONTRAST;
    ioctl(vfd, VIDIOC_G_CTRL, &ctrl);
    vpict.contrast = ctrl.value;

    ctrl.id=V4L2_CID_BRIGHTNESS;
    ioctl(vfd, VIDIOC_G_CTRL, &ctrl);
    vpict.brightness = ctrl.value;

    ctrl.id=V4L2_CID_HUE;
    ioctl(vfd, VIDIOC_G_CTRL, &ctrl);
    vpict.hue = ctrl.value;

    ctrl.id=V4L2_CID_SATURATION;
    ioctl(vfd, VIDIOC_G_CTRL, &ctrl);
    vpict.colour = ctrl.value;
  } else
    ioctl(vfd,VIDIOCGPICT, &vpict);
}


void set_tv_picture_ctrl()
{
  if (b_v4l2) {
    struct v4l2_control ctrl;

    ctrl.id=V4L2_CID_CONTRAST;
    ctrl.value = vpict.contrast;
    ioctl(vfd, VIDIOC_S_CTRL, &ctrl);

    ctrl.id=V4L2_CID_BRIGHTNESS;
    ctrl.value = vpict.brightness;
    ioctl(vfd, VIDIOC_S_CTRL, &ctrl);

    ctrl.id=V4L2_CID_HUE;
    ctrl.value = vpict.hue;
    ioctl(vfd, VIDIOC_S_CTRL, &ctrl);

    ctrl.id=V4L2_CID_SATURATION;
    ctrl.value = vpict.colour;
    ioctl(vfd, VIDIOC_S_CTRL, &ctrl);
  } else
    ioctl(vfd,VIDIOCSPICT, &vpict);
}

void set_pic_brightness(__s32 val)
{
  vpict.brightness = val;

  if (b_v4l2) {
    struct v4l2_control ctrl;

    ctrl.id=V4L2_CID_BRIGHTNESS;
    ctrl.value=val;
    ioctl(vfd, VIDIOC_S_CTRL, &ctrl);
  } else
    ioctl(vfd,VIDIOCSPICT, &vpict);
}


void set_pic_contrast(__s32 val)
{
  vpict.contrast = val;

  if (b_v4l2) {
    struct v4l2_control ctrl;

    ctrl.id=V4L2_CID_CONTRAST;
    ctrl.value=val;
    ioctl(vfd, VIDIOC_S_CTRL, &ctrl);
  } else
    ioctl(vfd,VIDIOCSPICT, &vpict);
}


void set_pic_hue(__s32 val)
{
    struct v4l2_control ctrl;

    ctrl.id=V4L2_CID_HUE;
    ctrl.value=val;
    vpict.hue = val;
    ioctl(vfd, VIDIOC_S_CTRL, &ctrl);
}


void set_pic_saturation(__s32 val)
{
    struct v4l2_control ctrl;

    ctrl.id=V4L2_CID_SATURATION;
    ctrl.value=val;
    vpict.colour = val;
    ioctl(vfd, VIDIOC_S_CTRL, &ctrl);
}


void get_tv_audio_ctrl(struct video_audio *ta)
{
  ioctl(vfd, VIDIOCGAUDIO, ta);
}


void set_tv_audio_ctrl(struct video_audio *ta)
{
  ioctl(vfd, VIDIOCSAUDIO, ta);
}


void set_tv_standard(TV_STANDARD *ptv_standard)
{
  extern TV_STANDARD *pcurrent_tv_std;

  if (!ptv_standard)
    ptv_standard = pcurrent_tv_std;

  dbg("set_tv_standard %s\n", ptv_standard->name);

  if (b_v4l2) {
    // Most tv card only support one standard, v4l2 should initialize it to a default vaule.
    v4l2_std_id stdid = ptv_standard->tv_standard_v4l2;
    dbg("default_tv_standard:%ld\n", stdid);
    ioctl(vfd, VIDIOC_S_STD, &stdid);
  } else {
    struct video_tuner tuner;

    bzero(&tuner, sizeof(tuner));
    ioctl(vfd, VIDIOCGTUNER, &tuner);

    tuner.flags &= ~(VIDEO_TUNER_PAL|VIDEO_TUNER_NTSC|VIDEO_TUNER_SECAM);
    switch (ptv_standard->tv_standard) {
      case VIDEO_MODE_PAL:
        tuner.flags|=VIDEO_TUNER_PAL;
        break;
      case VIDEO_MODE_NTSC:
        tuner.flags|=VIDEO_TUNER_NTSC;
        break;
      case VIDEO_MODE_SECAM:
        tuner.flags|=VIDEO_TUNER_SECAM;
        break;
    }

    tuner.mode = ptv_standard->tv_standard;
    ioctl(vfd, VIDIOCSTUNER, &tuner);
  }

  tv_pic_off();
  usleep(100000);
  tv_pic_on();
}


void v4l2_detect()
{
  dbg("v4l/v4l2 detection\n");

  struct v4l2_capability cap;

  if (ioctl(vfd, VIDIOC_QUERYCAP, &cap) == 0) {
    b_v4l2 = 1;

    if (fftv_mode & FFTV_MODE_TV)  {
      int fftv_system_chanlists_idx();

      if (!(cap.capabilities & V4L2_CAP_VIDEO_OVERLAY)) {
        tv_win_disp_mode = TV_GRAB_XV | TV_GRAB_XV_RECORD;
        dbg("Your v4l2 TV card doesn't support OVERLAY mode, use CAPTURE instead");
      }

      fftv_system_chanlists_idx();
    }

    dbg("........... v4l2 interface will be used ...........\n");

    return;
  }

  struct video_capability vcap;


  if (ioctl(vfd, VIDIOCGCAP, &vcap)==0) { // use V4L first

    b_v4l2 = 0;

    if (fftv_mode & FFTV_MODE_TV)  {
      int fftv_system_chanlists_idx();

      if (!(vcap.type & VID_TYPE_OVERLAY)) {
        tv_win_disp_mode = TV_GRAB_XV | TV_GRAB_XV_RECORD;
        dbg("Your v4l TV card doesn't support OVERLAY mode, use CAPTURE instead");
      }

      fftv_system_chanlists_idx();
    }

    dbg("............ 'v4l' interface will be used ...........\n");

    return;
  }
}


void tv_audio_on()
{
  if (b_v4l2) {
    struct v4l2_control ctl;

    ctl.id = V4L2_CID_AUDIO_MUTE;
    ctl.value = 0;
    ioctl(vfd, VIDIOC_S_CTRL, &ctl);
  } else {
    struct video_audio ta;

    bzero(&ta,sizeof(ta));
    get_tv_audio_ctrl(&ta);
    ta.flags &= ~VIDEO_AUDIO_MUTE;
    set_tv_audio_ctrl(&ta);
  }
}

void tv_audio_off()
{

  if (b_v4l2) {
    struct v4l2_control ctl;

    ctl.id = V4L2_CID_AUDIO_MUTE;
    ctl.value = 1;
    ioctl(vfd, VIDIOC_S_CTRL, &ctl);
  } else {
    struct video_audio ta;

    bzero(&ta,sizeof(ta));
    get_tv_audio_ctrl(&ta);
    ta.flags |= VIDEO_AUDIO_MUTE;
    set_tv_audio_ctrl(&ta);
  }
}

void tv_pic_on()
{
  if (tv_win_refresh_mode == TV_WIN_REFRESH_GRAB)
    return;
//  dbg("tv_pic_on .........................\n");
  if (b_v4l2)
    ioctl(vfd, VIDIOC_OVERLAY, &one);
  else
    ioctl(vfd, VIDIOCCAPTURE, &one);
}

void tv_pic_off()
{
  if (b_v4l2)
    ioctl(vfd, VIDIOC_OVERLAY, &zero);
  else
    ioctl(vfd, VIDIOCCAPTURE, &zero);
}

void get_tv_freq(__u32 *freq)
{
  if (b_v4l2) {
    struct v4l2_frequency vfreq;
    ioctl(vfd, VIDIOC_G_FREQUENCY, &vfreq);
    *freq = vfreq.frequency;
  } else {
    unsigned long freql = (unsigned long)*freq;
    ioctl(vfd, VIDIOCGFREQ, &freql);
  }
}

void set_tv_freq(__u32 *freq)
{
//  dbg("set freq %d\n", *freq);

  if (b_v4l2) {
    struct v4l2_frequency vfreq;
    vfreq.tuner = 0;
    vfreq.frequency = *freq;
    vfreq.type = (fftv_mode & FFTV_MODE_RADIO) ? V4L2_TUNER_RADIO : V4L2_TUNER_ANALOG_TV;
    ioctl(vfd, VIDIOC_S_FREQUENCY, &vfreq);
  } else {
    unsigned long freql = (unsigned long)*freq;
    ioctl(vfd, VIDIOCSFREQ, &freql);
  }
}

int tv_tuner_signal()
{
  if (b_v4l2) {
    struct v4l2_tuner tuner;

    bzero(&tuner,sizeof(tuner));
    if (ioctl(vfd,VIDIOC_G_TUNER,&tuner) == -1)
        return 0;
    return tuner.signal;
  } else {
    struct video_tuner tuner;

    bzero(&tuner,sizeof(tuner));

#if 0
    if (ioctl(vfd,VIDIOCGTUNER,&tuner) < 0)
      return 0;
#else
    ioctl(vfd,VIDIOCGTUNER,&tuner);
#endif
    return tuner.signal;
  }
}

