#include <stdio.h>
#include <sys/types.h>
#include <linux/videodev.h>
#include <sys/ioctl.h>
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include "fftv.h"

extern int b_v4l2;
static int active_vid_src_index;

#define MAX_VIDEO_SOURCE (16)

static char *vid_source_name[MAX_VIDEO_SOURCE];
static int vid_source_nameN;

static void init_vid_source_name_list()
{
  int i;

  if (vid_source_name[0])
    return;

  if (b_v4l2) {
    struct v4l2_input vinp;

    for(i=0; i < MAX_VIDEO_SOURCE; i++) {
      vinp.index = i;
      if (ioctl(vfd, VIDIOC_ENUMINPUT, &vinp) == -1)
        break;
      vid_source_name[vid_source_nameN++] = strdup(vinp.name);
    }
  } else {
    struct video_capability vid_cap;

    if (ioctl(vfd,VIDIOCGCAP,&vid_cap) == -1) {
      fprintf(stderr,"ioctl(h->fd,VIDIOCGCAP,&capN failed");
    }

    for (i = 0; i < vid_cap.channels; i++) {
        struct video_channel vidcha;

        vidcha.channel = i;
        ioctl(vfd,VIDIOCGCHAN,&vidcha);

        vid_source_name[vid_source_nameN++] = strdup(vidcha.name);
    }
  }
}


static char *tv_video_source_file_name()
{
  static char filename[256];

  get_fftv_states_dir(filename);
  strcat(filename,"/tv-video-source");

  return filename;
}

static void save_video_source()
{
  char *file = tv_video_source_file_name();

  FILE *fp;

  if ((fp=fopen(file, "w"))==NULL) {
    p_err("cannot write to file %s\n", file);
  }

  fprintf(fp,"%s\n", vid_source_name[active_vid_src_index]);
  fclose(fp);
}



int setVideoSource(char *vidname)
{
    int i;

    init_vid_source_name_list();
    if (!vid_source_nameN)
      p_err("no video source found");

    if (b_v4l2) {
      for(i=0; i < vid_source_nameN; i++) {

        dbg("%d %s\n", i, vid_source_name[i]);

        if (!strcmp(vid_source_name[i], vidname)) {
          active_vid_src_index = i;
          if (ioctl(vfd, VIDIOC_S_INPUT, &i) == -1)
            p_err("video source could not be set to %s", vidname);
          return -1;
        }
      }

    } else {
      for (i = 0; i < vid_source_nameN; i++) {
          struct video_channel vidcha;

          bzero(&vidcha,sizeof(vidcha));
          vidcha.channel = i;
          ioctl(vfd,VIDIOCGCHAN,&vidcha);

          if (!strcmp(vidcha.name,vidname)) {
            vidcha.norm=1;
            ioctl(vfd,VIDIOCSCHAN,&vidcha);
            active_vid_src_index = i;
            return -1;
          }
      }
    }

    dbg("cannot match the video source name %s",vidname);
}


void load_video_source()
{
  char *file = tv_video_source_file_name();

  FILE *fp;

  if ((fp=fopen(file, "r"))==NULL) {
    return;
  }

  dbg("load_tv_standard: %s\n", file);

  char aa[128];

  fscanf(fp,"%s", aa);

  fclose(fp);

  setVideoSource(aa);
}

void any_video_source(char *vidname)
{
    init_vid_source_name_list();

    if (!vid_source_nameN)
      p_err("no video source found");

    if (setVideoSource(vidname) < 0) {
      setVideoSource(vid_source_name[0]);
    }
}

static GtkWidget *vid_src_window = NULL;

static int new_select_idx;

static gboolean close_application( GtkWidget *widget, GdkEvent  *event,
                                   gpointer   data )
{
  active_vid_src_index = (int) new_select_idx;

  setVideoSource(vid_source_name[new_select_idx]);
  save_video_source();

  gtk_widget_destroy(vid_src_window);
  vid_src_window = NULL;

  return TRUE;
}


static void callback_button_clicked( GtkWidget *widget, gpointer   data)
{
   new_select_idx = GPOINTER_TO_INT(data);
}

extern GtkWidget *tv_ui_window;
void align_with_ui_window(GtkWidget *win);

void create_vid_src_window()
{
    GtkWidget *box1;
    GtkWidget *box2;
    GtkWidget *button, *button_close;
    GtkWidget *separator;
    GtkWidget *frame;

    init_vid_source_name_list();

    new_select_idx = active_vid_src_index;

    vid_src_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    align_with_ui_window(vid_src_window);

    g_signal_connect (G_OBJECT (vid_src_window), "delete_event",
                      G_CALLBACK (close_application),
                      NULL);

    gtk_window_set_title (GTK_WINDOW (vid_src_window), "TV system");
    gtk_container_set_border_width (GTK_CONTAINER (vid_src_window), 3);

    frame = gtk_frame_new("Select a video source");
    gtk_container_add (GTK_CONTAINER (vid_src_window), frame);

    box1 = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (frame), box1);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);

    int i;
    GSList *group=NULL;

    for(i=0; i < vid_source_nameN; i++) {

      button = gtk_radio_button_new_with_label (group, vid_source_name[i]);
      gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);

      group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));

      g_signal_connect (G_OBJECT (button), "clicked",
         G_CALLBACK (callback_button_clicked), (gpointer) i);

      if (i==active_vid_src_index)
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);

    }

    separator = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);

    button_close =gtk_button_new_from_stock (GTK_STOCK_OK);

    g_signal_connect_swapped (G_OBJECT (button_close), "clicked",
                              G_CALLBACK (close_application),
                              G_OBJECT (vid_src_window));
    gtk_box_pack_start (GTK_BOX (box2), button_close, TRUE, TRUE, 0);
    GTK_WIDGET_SET_FLAGS (button_close, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (button_close);

    gtk_widget_show_all (vid_src_window);

    return;
}
